@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* test.c: test builtin */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -630,6 +630,11 @@ enum filecmp compare_files(const wchar_t *left, const wchar_t *right)
 	return FC_OLDER;
     else if (sl.st_mtimensec > sr.st_mtimensec)
 	return FC_NEWER;
+#elif HAVE___ST_MTIMENSEC
+    else if (sl.__st_mtimensec < sr.__st_mtimensec)
+	return FC_OLDER;
+    else if (sl.__st_mtimensec > sr.__st_mtimensec)
+	return FC_NEWER;
 #endif
     else
 	return FC_SAME;
