@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* common.h: defines symbols common to all sources. */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,17 +25,14 @@
 # include "config.h"
 #endif
 
-#ifndef _POSIX_C_SOURCE
-# define _POSIX_C_SOURCE 200112L
-#endif
-#ifndef _XOPEN_SOURCE
-# define _XOPEN_SOURCE 600
-#endif
-
 #ifndef __GNUC__
 # define __attribute__(ignore)
 #endif
 
+#ifdef __CYGWIN__
+# undef __STRICT_ANSI__
+#endif
+
 #if HAVE_GETTEXT
 # define gt(MSGID)  ((const char *) gettext(MSGID))
 # define Ngt(MSGID) MSGID
