@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* display.c: display control */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -319,6 +319,7 @@ static struct {
     int width;      /* width of right prompt on screen */
 } rprompt;
 
+/* The value of the styler prompt where escape sequences have been processed. */
 static struct {
     char *value;
     size_t length;  /* number of bytes in `value' */
@@ -330,24 +331,26 @@ static struct {
 void le_display_init(struct promptset_T prompt_)
 {
     prompt = prompt_;
-
-    currentp.line = currentp.column = 0;
 }
 
-/* Finalizes the display module.
+/* Updates the prompt and the edit line, clears the candidate area, and leave
+ * the cursor at a blank line.
  * Must be called before `le_editing_finalize'. */
 void le_display_finalize(void)
 {
     assert(le_search_buffer.contents == NULL);
 
-    le_main_index = le_main_buffer.length;
-    le_display_update();
+    le_display_update(false);
 
     lebuf_print_sgr0();
-    if (lebuf.pos.column != 0 || lebuf.pos.line == 0)
-	lebuf_print_nel();
-    while (lebuf.pos.line <= rprompt_line)
+    if (rprompt_line >= 0) {
+	go_to((le_pos_T) { rprompt_line, le_columns - 1 });
 	lebuf_print_nel();
+    } else {
+	go_to_index(le_main_buffer.length);
+	if (lebuf.pos.column != 0 || lebuf.pos.line == 0)
+	    lebuf_print_nel();
+    }
 
     clean_up();
 }
@@ -420,7 +423,6 @@ void clear_editline(void)
     assert(lebuf.pos.line > editbasep.line
 	    || (lebuf.pos.line == editbasep.line
 		&& lebuf.pos.column >= editbasep.column));
-    assert(lebuf.pos.line <= last_edit_line);
 
     le_pos_T save_pos = lebuf.pos;
 
@@ -437,10 +439,11 @@ void clear_editline(void)
     go_to(save_pos);
 }
 
-/* (Re)prints the display appropriately and moves the cursor to the proper
- * position. The print buffer must not have been initialized.
+/* (Re)prints the display appropriately and, if `cursor' is true, moves the
+ * cursor to the proper position.
+ * The print buffer must not have been initialized.
  * The output is sent to the print buffer. */
-void le_display_update(void)
+void le_display_update(bool cursor)
 {
     if (!display_active) {
 	display_active = true;
@@ -476,6 +479,7 @@ void le_display_update(void)
 	lebuf_init((le_pos_T) { 0, 0 });
 	maybe_print_promptsp();
 	lebuf_print_prompt(prompt.main);
+	fillip_cursor();
 	editbasep = lebuf.pos;
     } else {
 	lebuf_init(currentp);
@@ -497,7 +501,8 @@ void le_display_update(void)
 
     /* set cursor position */
     assert(le_main_index <= le_main_buffer.length);
-    go_to_index(le_main_index);
+    if (cursor)
+	go_to_index(le_main_index);
 }
 
 /* Prints a dummy string that moves the cursor to the first column of the next
@@ -509,7 +514,8 @@ void maybe_print_promptsp(void)
 	lebuf_print_smso();
 	lebuf_putchar('$');
 	lebuf_print_sgr0();
-	for (int i = le_ti_xenl ? 1 : 2; i < le_columns; i++)
+	int count = le_columns - (le_ti_xenl ? 1 : 2);
+	while (--count >= 0)
 	    lebuf_putchar(' ');
 	lebuf_print_cr();
 	lebuf_print_ed();
@@ -565,13 +571,17 @@ void update_editline(void)
     cursor_positions[index] = lebuf.pos.line * le_columns + lebuf.pos.column;
 
     fillip_cursor();
-    last_edit_line = lebuf.pos.line;
+
+    last_edit_line =
+	(lebuf.pos.line >= rprompt_line) ? lebuf.pos.line : rprompt_line;
 
     /* clear the right prompt if the edit line reaches it. */
     if (rprompt_line == lebuf.pos.line
 	    && lebuf.pos.column > le_columns - rprompt.width - 2) {
 	lebuf_print_el();
 	rprompt_line = -1;
+    } else if (rprompt_line < lebuf.pos.line) {
+	rprompt_line = -1;
     }
 }
 
@@ -593,6 +603,8 @@ void update_right_prompt(void)
 	return;
     if (rprompt.width == 0)
 	return;
+    if (le_columns - rprompt.width - 2 < 0)
+	return;
     int c = cursor_positions[le_main_buffer.length] % le_columns;
     bool has_enough_room = (c <= le_columns - rprompt.width - 2);
     if (!has_enough_room && !shopt_le_alwaysrp)
@@ -635,6 +647,8 @@ void print_search(void)
 
     const char *text;
     switch (le_search_type) {
+	case SEARCH_PREFIX:
+	    assert(false);
 	case SEARCH_VI:
 	    switch (le_search_direction) {
 		case FORWARD:   lebuf_putwchar(L'?', false);  break;
