@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* editing.c: main editing module */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "../common.h"
 #include <assert.h>
+#include <errno.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -266,10 +267,11 @@ static struct xwcsrange get_prev_bigword(
 static void replace_horizontal_space(bool deleteafter, const wchar_t *s)
     __attribute__((nonnull));
 
-static void go_to_history_absolute(const histentry_T *e, bool cursorend)
+static void go_to_history_absolute(
+	const histentry_T *e, enum le_search_type curpos)
     __attribute__((nonnull));
-static void go_to_history_relative(int offset, bool cursorend);
-static void go_to_history(const histentry_T *e, bool cursorend)
+static void go_to_history_relative(int offset, enum le_search_type curpos);
+static void go_to_history(const histentry_T *e, enum le_search_type curpos)
     __attribute__((nonnull));
 
 static bool need_update_last_search_value(void)
@@ -279,6 +281,9 @@ static void perform_search(const wchar_t *pattern,
 	enum le_search_direction dir, enum le_search_type type)
     __attribute__((nonnull));
 static void search_again(enum le_search_direction dir);
+static void beginning_search(enum le_search_direction dir);
+static inline bool beginning_search_check_go_to_history(const wchar_t *prefix)
+    __attribute__((nonnull,pure));
 
 #define ALERT_AND_RETURN_IF_PENDING                     \
     do if (state.pending_command_motion != MEC_NONE)    \
@@ -710,19 +715,26 @@ void cmd_bol_or_digit(wchar_t c)
 }
 
 /* Accepts the current line.
- * `le_state' is set to LE_STATE_DONE to trigger the termination of lineedit. */
+ * `le_editstate' is set to LE_EDITSTATE_DONE to induce lineedit to terminate.
+ * If history search is currently active, the search result is accepted. If the
+ * search was failing, the line is not accepted. */
 void cmd_accept_line(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
 
-    cmd_srch_accept_search(L'\0');
-    le_editstate = LE_EDITSTATE_DONE;
-    reset_state();
+    if (le_search_buffer.contents == NULL) {
+	le_editstate = LE_EDITSTATE_DONE;
+	reset_state();
+    } else {
+	if (le_search_result != Histlist)
+	    le_editstate = LE_EDITSTATE_DONE;
+	cmd_srch_accept_search(L'\0');
+    }
 }
 
 /* Aborts the current line.
- * `le_state' is set to LE_STATE_INTERRUPTED to trigger the termination of
- * lineedit. */
+ * `le_editstate' is set to LE_EDITSTATE_INTERRUPTED to induce lineedit to
+ * terminate. */
 void cmd_abort_line(wchar_t c __attribute__((unused)))
 {
     cmd_srch_abort_search(L'\0');
@@ -730,7 +742,7 @@ void cmd_abort_line(wchar_t c __attribute__((unused)))
     reset_state();
 }
 
-/* Sets `le_state' to LE_STATE_ERROR.
+/* Sets `le_editstate' to LE_EDITSTATE_ERROR.
  * Lineedit will return EOF. */
 void cmd_eof(wchar_t c __attribute__((unused)))
 {
@@ -741,8 +753,8 @@ void cmd_eof(wchar_t c __attribute__((unused)))
     reset_state();
 }
 
-/* If the edit line is empty, sets `le_state' to LE_STATE_ERROR (return EOF).
- * Otherwise, alerts. */
+/* If the edit line is empty, sets `le_editstate' to LE_EDITSTATE_ERROR (return
+ * EOF). Otherwise, alerts. */
 void cmd_eof_if_empty(wchar_t c __attribute__((unused)))
 {
     if (le_main_buffer.length == 0)
@@ -751,8 +763,8 @@ void cmd_eof_if_empty(wchar_t c __attribute__((unused)))
 	cmd_alert(L'\0');
 }
 
-/* If the edit line is empty, sets `le_state' to LE_STATE_ERROR (return EOF).
- * Otherwise, deletes the character under the cursor. */
+/* If the edit line is empty, sets `le_editstate' to LE_EDITSTATE_ERROR (return
+ * EOF). Otherwise, deletes the character under the cursor. */
 void cmd_eof_or_delete(wchar_t c __attribute__((unused)))
 {
     if (le_main_buffer.length == 0)
@@ -2401,6 +2413,7 @@ void vi_exec_alias(wchar_t c)
 }
 
 /* Invokes an external command to edit the current line and accepts the result.
+ * If the count is set, goes to the `count'th history entry and edit it.
  * If the editor returns a non-zero status, the line is not accepted. */
 /* cf. history.c:fc_edit_and_exec_entries */
 void cmd_vi_edit_and_accept(wchar_t c __attribute__((unused)))
@@ -2413,26 +2426,41 @@ void cmd_vi_edit_and_accept(wchar_t c __attribute__((unused)))
     pid_t cpid;
     int savelaststatus;
 
+    if (state.count.sign != 0) {
+	int num = get_count(0);
+	if (num < 0)
+	    goto error0;
+	const histentry_T *e = get_history_entry((unsigned) num);
+	if (e == NULL)
+	    goto error0;
+	go_to_history(e, SEARCH_VI);
+    }
+
+    le_suspend_readline();
     fd = create_temporary_file(&tempfile, S_IRUSR | S_IWUSR);
     if (fd < 0) {
-	cmd_alert(L'\0');
-	return;
+	xerror(errno, Ngt("cannot create temporary file to edit history"));
+	goto error1;
     }
     f = fdopen(fd, "w");
     if (f == NULL) {
+	xerror(errno, Ngt("cannot open temporary file to edit history"));
 	xclose(fd);
-	cmd_alert(L'\0');
-	return;
+	goto error2;
     }
-    le_suspend_readline();
 
     savelaststatus = laststatus;
     cpid = fork_and_reset(0, true, 0);
     if (cpid < 0) { // fork failed
+	xerror(0, Ngt("cannot invoke editor to edit history"));
 	fclose(f);
-	unlink(tempfile);
+	if (unlink(tempfile) < 0)
+	    xerror(errno, Ngt("cannot remove temporary file `%s'"), tempfile);
+error2:
 	free(tempfile);
+error1:
 	le_resume_readline();
+error0:
 	cmd_alert(L'\0');
     } else if (cpid > 0) {  // parent process
 	fclose(f);
@@ -2743,28 +2771,55 @@ void cmd_emacs_search_backward(wchar_t c __attribute__((unused)))
 /* Goes to the oldest history entry.
  * If the count is specified, goes to the history entry whose number is count.
  * If the specified entry is not found, the terminal is alerted.
- * The cursor is put at the beginning of line. */
+ * The cursor position is not changed. */
 void cmd_oldest_history(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(histlist.Oldest, false);
+    go_to_history_absolute(histlist.Oldest, SEARCH_PREFIX);
 }
 
 /* Goes to the newest history entry.
  * If the count is specified, goes to the history entry whose number is count.
  * If the specified entry is not found, the terminal is alerted.
- * The cursor is put at the beginning of line. */
+ * The cursor position is not changed. */
 void cmd_newest_history(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(histlist.Newest, false);
+    go_to_history_absolute(histlist.Newest, SEARCH_PREFIX);
+}
+
+/* Goes to the newest history entry.
+ * If the count is specified, goes to the history entry whose number is count.
+ * If the specified entry is not found, the terminal is alerted.
+ * The cursor position is not changed. */
+void cmd_return_history(wchar_t c __attribute__((unused)))
+{
+    go_to_history_absolute(Histlist, SEARCH_PREFIX);
+}
+
+/* Goes to the oldest history entry.
+ * If the count is specified, goes to the history entry whose number is count.
+ * If the specified entry is not found, the terminal is alerted.
+ * The cursor is put at the beginning of line. */
+void cmd_oldest_history_bol(wchar_t c __attribute__((unused)))
+{
+    go_to_history_absolute(histlist.Oldest, SEARCH_VI);
 }
 
 /* Goes to the newest history entry.
  * If the count is specified, goes to the history entry whose number is count.
  * If the specified entry is not found, the terminal is alerted.
  * The cursor is put at the beginning of line. */
-void cmd_return_history(wchar_t c __attribute__((unused)))
+void cmd_newest_history_bol(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(Histlist, false);
+    go_to_history_absolute(histlist.Newest, SEARCH_VI);
+}
+
+/* Goes to the newest history entry.
+ * If the count is specified, goes to the history entry whose number is count.
+ * If the specified entry is not found, the terminal is alerted.
+ * The cursor is put at the beginning of line. */
+void cmd_return_history_bol(wchar_t c __attribute__((unused)))
+{
+    go_to_history_absolute(Histlist, SEARCH_VI);
 }
 
 /* Goes to the oldest history entry.
@@ -2773,7 +2828,7 @@ void cmd_return_history(wchar_t c __attribute__((unused)))
  * The cursor is put at the end of line. */
 void cmd_oldest_history_eol(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(histlist.Oldest, true);
+    go_to_history_absolute(histlist.Oldest, SEARCH_EMACS);
 }
 
 /* Goes to the newest history entry.
@@ -2782,7 +2837,7 @@ void cmd_oldest_history_eol(wchar_t c __attribute__((unused)))
  * The cursor is put at the end of line. */
 void cmd_newest_history_eol(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(histlist.Newest, true);
+    go_to_history_absolute(histlist.Newest, SEARCH_EMACS);
 }
 
 /* Goes to the newest history entry.
@@ -2791,15 +2846,14 @@ void cmd_newest_history_eol(wchar_t c __attribute__((unused)))
  * The cursor is put at the end of line. */
 void cmd_return_history_eol(wchar_t c __attribute__((unused)))
 {
-    go_to_history_absolute(Histlist, true);
+    go_to_history_absolute(Histlist, SEARCH_EMACS);
 }
 
 /* Goes to the specified history entry.
  * If the count is specified, goes to the history entry whose number is count.
  * If the specified entry is not found, the terminal is alerted.
- * If `cursorend' is true, the cursor is put at the end of line; otherwise, at
- * the beginning of line. */
-void go_to_history_absolute(const histentry_T *e, bool cursorend)
+ * See `go_to_history' for the meaning of `curpos'. */
+void go_to_history_absolute(const histentry_T *e, enum le_search_type curpos)
 {
     ALERT_AND_RETURN_IF_PENDING;
 
@@ -2814,7 +2868,7 @@ void go_to_history_absolute(const histentry_T *e, bool cursorend)
 	if (e == NULL)
 	    goto alert;
     }
-    go_to_history(e, cursorend);
+    go_to_history(e, curpos);
     reset_state();
     return;
 
@@ -2824,19 +2878,35 @@ alert:
 }
 
 /* Goes to the `count'th next history entry.
- * The cursor is put at the beginning of line. */
+ * The cursor position is not changed. */
 void cmd_next_history(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
-    go_to_history_relative(get_count(1), false);
+    go_to_history_relative(get_count(1), SEARCH_PREFIX);
+}
+
+/* Goes to the `count'th previous history entry.
+ * The cursor position is not changed. */
+void cmd_prev_history(wchar_t c __attribute__((unused)))
+{
+    ALERT_AND_RETURN_IF_PENDING;
+    go_to_history_relative(-get_count(1), SEARCH_PREFIX);
+}
+
+/* Goes to the `count'th next history entry.
+ * The cursor is put at the beginning of line. */
+void cmd_next_history_bol(wchar_t c __attribute__((unused)))
+{
+    ALERT_AND_RETURN_IF_PENDING;
+    go_to_history_relative(get_count(1), SEARCH_VI);
 }
 
 /* Goes to the `count'th previous history entry.
  * The cursor is put at the beginning of line. */
-void cmd_prev_history(wchar_t c __attribute__((unused)))
+void cmd_prev_history_bol(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
-    go_to_history_relative(-get_count(1), false);
+    go_to_history_relative(-get_count(1), SEARCH_VI);
 }
 
 /* Goes to the `count'th next history entry.
@@ -2844,7 +2914,7 @@ void cmd_prev_history(wchar_t c __attribute__((unused)))
 void cmd_next_history_eol(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
-    go_to_history_relative(get_count(1), true);
+    go_to_history_relative(get_count(1), SEARCH_EMACS);
 }
 
 /* Goes to the `count'th previous history entry.
@@ -2852,13 +2922,12 @@ void cmd_next_history_eol(wchar_t c __attribute__((unused)))
 void cmd_prev_history_eol(wchar_t c __attribute__((unused)))
 {
     ALERT_AND_RETURN_IF_PENDING;
-    go_to_history_relative(-get_count(1), true);
+    go_to_history_relative(-get_count(1), SEARCH_EMACS);
 }
 
 /* Goes to the `offset'th next history entry.
- * If `cursorend' is true, the cursor is put at the end of line; otherwise, at
- * the beginning of line. */
-void go_to_history_relative(int offset, bool cursorend)
+ * See `go_to_history' for the meaning of `curpos'. */
+void go_to_history_relative(int offset, enum le_search_type curpos)
 {
     const histentry_T *e = main_history_entry;
     if (offset > 0) {
@@ -2874,18 +2943,21 @@ void go_to_history_relative(int offset, bool cursorend)
 		goto alert;
 	} while (++offset < 0);
     }
-    go_to_history(e, cursorend);
+    go_to_history(e, curpos);
     reset_state();
     return;
+
 alert:
     cmd_alert(L'\0');
     return;
 }
 
 /* Sets the value of the specified history entry to the main buffer.
- * If `cursorend' is true, the cursor is put at the end of line; otherwise, at
- * the beginning of line. */
-void go_to_history(const histentry_T *e, bool cursorend)
+ * The value of `curpos' specifies where the cursor is left:
+ *  SEARCH_PREFIX: the current position (unless it exceeds the buffer length)
+ *  SEARCH_VI:     the beginning of the buffer
+ *  SEARCH_EMACS:  the end of the buffer */
+void go_to_history(const histentry_T *e, enum le_search_type curpos)
 {
     maybe_save_undo_history();
 
@@ -2896,11 +2968,21 @@ void go_to_history(const histentry_T *e, bool cursorend)
 	wb_cat(&le_main_buffer, h->contents);
 	assert(h->index <= le_main_buffer.length);
 	le_main_index = h->index;
-    } else if (e != Histlist) {
-	wb_mbscat(&le_main_buffer, e->value);
-	le_main_index = cursorend ? le_main_buffer.length : 0;
     } else {
-	le_main_index = 0;
+	if (e != Histlist)
+	    wb_mbscat(&le_main_buffer, e->value);
+	switch (curpos) {
+	    case SEARCH_PREFIX:
+		if (le_main_index > le_main_buffer.length)
+		    le_main_index = le_main_buffer.length;
+		break;
+	    case SEARCH_VI:
+		le_main_index = 0;
+		break;
+	    case SEARCH_EMACS:
+		le_main_index = le_main_buffer.length;
+		break;
+	}
     }
     main_history_entry = e;
     main_history_value = xwcsdup(le_main_buffer.contents);
@@ -2931,13 +3013,17 @@ void cmd_srch_backward_delete_char(wchar_t c __attribute__((unused)))
 	return;
     }
 
-    if (le_search_buffer.length == 0) switch (le_search_type) {
-	case SEARCH_VI:
-	    cmd_srch_abort_search(L'\0');
-	    return;
-	case SEARCH_EMACS:
-	    cmd_alert(L'\0');
-	    return;
+    if (le_search_buffer.length == 0) {
+	switch (le_search_type) {
+	    case SEARCH_VI:
+		cmd_srch_abort_search(L'\0');
+		return;
+	    case SEARCH_EMACS:
+		cmd_alert(L'\0');
+		return;
+	    default:
+		assert(false);
+	}
     }
 
     wb_remove(&le_search_buffer, le_search_buffer.length - 1, 1);
@@ -2968,7 +3054,7 @@ void cmd_srch_continue_forward(wchar_t c __attribute__((unused)))
 
     le_search_direction = FORWARD;
     if (le_search_result != Histlist)
-	go_to_history(le_search_result, le_search_type == SEARCH_EMACS);
+	go_to_history(le_search_result, le_search_type);
     update_search();
 }
 
@@ -2984,7 +3070,7 @@ void cmd_srch_continue_backward(wchar_t c __attribute__((unused)))
 
     le_search_direction = BACKWARD;
     if (le_search_result != Histlist)
-	go_to_history(le_search_result, le_search_type == SEARCH_EMACS);
+	go_to_history(le_search_result, le_search_type);
     update_search();
 }
 
@@ -3008,7 +3094,7 @@ void cmd_srch_accept_search(wchar_t c __attribute__((unused)))
     if (le_search_result == Histlist) {
 	cmd_alert(L'\0');
     } else {
-	go_to_history(le_search_result, le_search_type == SEARCH_EMACS);
+	go_to_history(le_search_result, le_search_type);
     }
     reset_state();
 }
@@ -3018,6 +3104,8 @@ void cmd_srch_accept_search(wchar_t c __attribute__((unused)))
 bool need_update_last_search_value(void)
 {
     switch (le_search_type) {
+	case SEARCH_PREFIX:
+	    break;
 	case SEARCH_VI:
 	    if (le_search_buffer.contents[0] == L'\0')
 		return false;
@@ -3050,6 +3138,8 @@ void update_search(void)
     const wchar_t *pattern = le_search_buffer.contents;
     if (pattern[0] == L'\0') {
 	switch (le_search_type) {
+	    case SEARCH_PREFIX:
+		break;
 	    case SEARCH_VI:
 		pattern = last_search.value;
 		if (pattern == NULL) {
@@ -3080,6 +3170,12 @@ void perform_search(const wchar_t *pattern,
 	goto done;
 
     switch (type) {
+	case SEARCH_PREFIX: {
+	    wchar_t *p = escape(pattern, NULL);
+	    xfnm = xfnm_compile(p, XFNM_HEADONLY);
+	    free(p);
+	    break;
+	}
 	case SEARCH_VI: {
 	    xfnmflags_T flags = 0;
 	    if (pattern[0] == L'^') {
@@ -3176,11 +3272,71 @@ void search_again(enum le_search_direction dir)
 	    cmd_alert(L'\0');
 	    break;
 	} else {
-	    go_to_history(le_search_result, false);
+	    go_to_history(le_search_result, last_search.type);
 	}
     }
     reset_state();
 }
 
+/* Searches the history in the forward direction for an entry that has a common
+ * prefix with the current buffer contents. The "prefix" is the first
+ * `le_main_index' characters of the buffer. */
+void cmd_beginning_search_forward(wchar_t c __attribute__((unused)))
+{
+    beginning_search(FORWARD);
+}
+
+/* Searches the history in the backward direction for an entry that has a common
+ * prefix with the current buffer contents. The "prefix" is the first
+ * `le_main_index' characters of the buffer. */
+void cmd_beginning_search_backward(wchar_t c __attribute__((unused)))
+{
+    beginning_search(BACKWARD);
+}
+
+/* Searches the history in the specified direction for an entry that has a
+ * common prefix with the current buffer contents. The "prefix" is the first
+ * `le_main_index' characters of the buffer. */
+void beginning_search(enum le_search_direction dir)
+{
+    ALERT_AND_RETURN_IF_PENDING;
+
+    int count = get_count(1);
+    if (count < 0) {
+	count = -count;
+	switch (dir) {
+	    case FORWARD:  dir = BACKWARD; break;
+	    case BACKWARD: dir = FORWARD;  break;
+	}
+    }
+
+    wchar_t *prefix = xwcsndup(le_main_buffer.contents, le_main_index);
+    while (--count >= 0) {
+	perform_search(prefix, dir, SEARCH_PREFIX);
+	if (le_search_result == Histlist) {
+	    if (dir == FORWARD && beginning_search_check_go_to_history(prefix))
+		go_to_history(le_search_result, SEARCH_PREFIX);
+	    else
+		cmd_alert(L'\0');
+	    break;
+	} else {
+	    go_to_history(le_search_result, SEARCH_PREFIX);
+	}
+    }
+    free(prefix);
+    reset_state();
+}
+
+bool beginning_search_check_go_to_history(const wchar_t *prefix)
+{
+    if (le_search_result == undo_history_entry
+	    && undo_index < undo_history.length) {
+	const struct undo_history *h = undo_history.contents[undo_index];
+	return matchwcsprefix(h->contents, prefix);
+    } else {
+	return false;
+    }
+}
+
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
