@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* mkcmds.c: outputs string for 'commands.in' contents */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -136,11 +136,16 @@ static char *commands[] = {
     "oldest-history",
     "newest-history",
     "return-history",
+    "oldest-history-bol",
+    "newest-history-bol",
+    "return-history-bol",
     "oldest-history-eol",
     "newest-history-eol",
     "return-history-eol",
     "next-history",
     "prev-history",
+    "next-history-bol",
+    "prev-history-bol",
     "next-history-eol",
     "prev-history-eol",
     "srch-self-insert",
@@ -154,6 +159,8 @@ static char *commands[] = {
     "search-again-rev",
     "search-again-forward",
     "search-again-backward",
+    "beginning-search-forward",
+    "beginning-search-backward",
 };
 
 int sorter(const void *p1, const void *p2)
