@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* variable.c: deals with shell variables and parameters */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -185,7 +185,7 @@ static environ_T *first_env;
 /* whether $RANDOM is functioning as a random number */
 static bool random_active;
 
-/* hashtable from function names (char *) to functions (function_T *). */
+/* hashtable from function names (wchar_t *) to functions (function_T *). */
 static hashtable_T functions;
 
 
@@ -752,7 +752,7 @@ void xtrace_array(const wchar_t *name, void *const *values)
 	    wb_wccat(buf, L' ');
 	}
     }
-    fputs(")", stderr);
+    wb_wccat(buf, L')');
 }
 
 /* Gets the value of the specified scalar variable.
